#include "fxlib.h"
#include "stdlib.h"
#include "MonochromeLib.h"
#include "functions.h"
#include "special_chars.h"
#include "ECode.h"
#define HEADER_SIZE	86


int F_get_contrast()
{
	Model model = modelDetector();
	
	switch (model) {
		case EMULATOR: return 0; break;
		case G35: return 162; break;
		case G75: return 168; break;
		case G85: return 170; break;
		case G85SD: return 170; break;
		case G95: return 168; break;
		case UNDEFINED: return 0; break;
	}
}

void F_bmp(int filename_int, int number, int x, int y, int width, int height, int mode)
{
	int file, size, pst;
	unsigned char *content, *sprite;
	char search[10] = {'\'','S','P','R','I','T','E'};
	unsigned const int length = F_real_size(width*height/8, 8);
	int n, i;
	
	FONTCHARACTER filename[] = {'\\','\\','f','l','s','0','\\','P','R','G','M','2','\\','0','0','0','0','.','g','1','m',0};
	for (i=0 ; i<4 ; i++) filename[13+i] = F_extract_digit(filename_int, 2*i);
	
	file = Bfile_OpenFile(filename, _OPENMODE_READ);
	if(file>=0) {
		size = Bfile_GetFileSize(file) - HEADER_SIZE;
		content = (char*)malloc(size);
		Bfile_ReadFile(file, content, size, HEADER_SIZE);
		Bfile_CloseFile(file);
	
		if(number>9) {
			search[8] = number%10+'0';
			number /= 10;
		}
		search[7] = number+'0';
		pst = F_search(content, &search, size);
		
		if(pst==-1) { // no sprite
			free(content);
			return; 
		}
		
		pst += 8;
		while (content[pst]!='{') {
			pst++;
			if(content[pst]=='\r' || pst>size) { // no sprite code
				free(content);
				return;
			}
		}
		pst++;
		
		sprite = (char*)malloc(length+1);	
		
		for (i=0 ; i<length ; i++) {
			while (content[pst]<'0' || content[pst]>'9') {
				pst++;
				if(content[pst]=='\r' || content[pst]=='}' || pst>size) { // sprite code is not full
					free(sprite);
					free(content);
					return;
				}
			}
			
			n = 0;
			while (content[pst]>='0' && content[pst]<='9' && pst<=size) {
				n = n*10+content[pst]-'0';
				pst++;
			}
			
			sprite[i] = n;
		}
		sprite[i] = '\0';
		
		switch (mode) {
			case 0: ML_bmp_or_cl(sprite, x, y, width, height); break;
			case 1: ML_bmp_and_cl(sprite, x, y, width, height); break;
			case 2: ML_bmp_xor_cl(sprite, x, y, width, height); break;
		}
		
		free(sprite);
		free(content);
	}
}

void F_text(int filename_int, int number, int x, int y, int mode, int mode2)
{
	int file, size, pst;
	unsigned char *content, *text;
	char search[8] = {'\'','T','E','X','T'};
	int length = 0;
	int i;
	
	FONTCHARACTER filename[] = {'\\','\\','f','l','s','0','\\','P','R','G','M','2','\\','0','0','0','0','.','g','1','m',0};
	for (i=0 ; i<4 ; i++) filename[13+i] = F_extract_digit(filename_int, 2*i);
	
	file = Bfile_OpenFile(filename, _OPENMODE_READ);
	if(file>=0) {
		size = Bfile_GetFileSize(file) - HEADER_SIZE;
		content = (char*)malloc(size);
		Bfile_ReadFile(file, content, size, HEADER_SIZE);
		Bfile_CloseFile(file);
	
		if(number>9) {
			search[6] = number%10+'0';
			number /= 10;
		}
		search[5] = number+'0';
		pst = F_search(content, &search, size);
		
		if(pst==-1) { // no text
			free(content);
			return; 
		}
		
		pst += 6;
		while (content[pst]!='"') {
			pst++;
			if(content[pst]=='\r' || pst>size) { // no text
				free(content);
				return;
			}
		}
		pst++;
		
		while (content[pst+length]!='"' && pst+length<=size) {
			length++;
		}
	
		text = (char*)malloc(length+1);	
		
		for (i=0 ; i<length ; i++) {
			switch(content[pst+i]) {
				case PLUS: text[i] = CHAR_MB1_PLUS; break;
				case MINUS: text[i] = CHAR_MB1_MINUS; break;
				
				default: text[i] = content[pst+i]; break;
			}
		}
		text[i] = '\0';
		
		switch (mode) {
			case 0: PrintXY(x, y, text, mode2); break;
			case 1:
				switch (mode2) {
					case 0: PrintMini(x, y, text, MINI_OVER); break;
					case 1: PrintMini(x, y, text, MINI_OR); break;
					case 2: PrintMini(x, y, text, MINI_REV); break;
					case 3: PrintMini(x, y, text, MINI_REVOR); break;
				} break;
		}
		
		free(text);
		free(content);
	}
}

void F_number(int n, int x, int y, int mode, int mode2)
{
	int i, length=0;
    char str[12] = {0};
    if(n) {
        if(n<0) {
            length++;
            str[0] = '-';
            n = -n;
        }
        for(i=n ; i ; i/=10) length++;
        for( ; n ; n/=10) str[--length] = n%10+'0';
    }
	else str[0] = '0';
	
	switch (mode) {
			case 0: PrintXY(x, y, str, mode2); break;
			case 1:
				switch (mode2) {
					case 0: PrintMini(x, y, str, MINI_OVER); break;
					case 1: PrintMini(x, y, str, MINI_OR); break;
					case 2: PrintMini(x, y, str, MINI_REV); break;
					case 3: PrintMini(x, y, str, MINI_REVOR); break;
				} break;
		}
}


void F_file_create(int filename_int, int size)
{
	int file, i;
	int init = 0;
	FONTCHARACTER filename[] = {'\\','\\','f','l','s','0','\\','P','R','G','M','2','\\','0','0','0','0','.','s','a','v',0};
	for (i=0 ; i<4 ; i++) filename[13+i] = F_extract_digit(filename_int, 2*i);
	
	Bfile_CreateFile(filename, 4*size);
	
	file = Bfile_OpenFile(filename, _OPENMODE_WRITE);
	if(file>=0) {
		for (i=0 ; i<size ; i++) Bfile_WriteFile(file, &init, 4);
		Bfile_CloseFile(file);
	}
}

int F_file_read(int filename_int, int position, int *byte)
{
	int file, i;
	int ret = 1;
	FONTCHARACTER filename[] = {'\\','\\','f','l','s','0','\\','P','R','G','M','2','\\','0','0','0','0','.','s','a','v',0};
	for (i=0 ; i<4 ; i++) filename[13+i] = F_extract_digit(filename_int, 2*i);
	
	file = Bfile_OpenFile(filename, _OPENMODE_READ);
	if(file>=0) {
		ret = 0;
		Bfile_ReadFile(file, byte, 4, 4*position);
		Bfile_CloseFile(file);
	}
	return ret;
}

void F_file_write(int filename_int, int position, int byte)
{
	int file, i;
	int size;
	unsigned char *content1, *content2;
	FONTCHARACTER filename[] = {'\\','\\','f','l','s','0','\\','P','R','G','M','2','\\','0','0','0','0','.','s','a','v',0};
	for (i=0 ; i<4 ; i++) filename[13+i] = F_extract_digit(filename_int, 2*i);
	
	file = Bfile_OpenFile(filename, _OPENMODE_READ);
	if(file>=0) {
		size = Bfile_GetFileSize(file);
		if(position>0) {
			content1 = (char*)malloc(4*position);
			Bfile_ReadFile(file, content1, 4*position, 0);
		}
		if(4*position+4<size) {
			content2 = (char*)malloc(size-(4*position+4));
			Bfile_ReadFile(file, content2, size-(4*position+4), 4*position+4);
		}
		Bfile_CloseFile(file);
	}
	
	Bfile_DeleteFile(filename);
	Bfile_CreateFile(filename, size);
	file = Bfile_OpenFile(filename, _OPENMODE_WRITE);
	if(file>=0) {
		
		if(position>0) Bfile_WriteFile(file, content1, 4*position);
		Bfile_WriteFile(file, &byte, 4);
		if(4*position+4<size) Bfile_WriteFile(file, content2, size-(4*position+4));
		
		Bfile_CloseFile(file);
	}
}

int F_key()
{
	unsigned int key = 0;
	key += IsKeyDown(KEY_CTRL_RIGHT);
	key += 10*IsKeyDown(KEY_CTRL_UP);
	key += 100*IsKeyDown(KEY_CTRL_DOWN);
	key += 1e3*IsKeyDown(KEY_CTRL_LEFT);
	key += 1e4*IsKeyDown(KEY_CTRL_ALPHA);
	key += 1e5*IsKeyDown(KEY_CTRL_SHIFT);
	return key;
}


int F_real_size(int nb, int d)
{
	int ret = nb;
	if(nb%d!=0) ret = nb+d-nb%d;
	return ret;
}

int F_extract_digit(int nb, int d)
{
	int ret, i;
	for (i=6 ; i>d ; i--) nb /= 10;
	ret = nb%100;
	return ret;
}

int F_search(char *content, char *search, int size)
{
	int ret, n, i;
	
	for(ret = 0 ; ret<=size ; ret++) {
		if(content[ret]==search[0]) {
			n = 1;
			for (i=1 ; search[i] ; i++) {
				if(content[ret+i] != search[i]) {
					n = 0;
					break;
				}
			}
			if(n) return ret;
		}
	}
	return -1;
}